@@ -1,13 +1,13 @@
 #' @title Statistical transformation for rasters
 #' @description Transforms raster to a specified statistical transformation 
 #' 
-#' @param x       raster class object
+#' @param x       A terra SpatRaster class object
 #' @param trans   Transformation method: "norm", "rstd", "std", "stretch", 
 #'                 "nl", "slog", "sr" (please see notes)
 #' @param smin    Minimum value for stretch 
 #' @param smax    Maximum value for stretch
 #' 
-#' @return raster class object of transformation
+#' @return A terra SpatRaster class object of specified transformation
 #'
 #' @description
 #' Transformation option details:
@@ -28,18 +28,19 @@
 #' 
 #' @examples 
 #' \donttest{
-#'   library(raster)
-#'   r <- raster(nrows=100, ncols=100, xmn=571823, xmx=616763, 
-#'               ymn=4423540, ymx=4453690)
-#'     r[] <- runif(ncell(r), 1000, 2500)
+#' library(terra)
+#' r <- rast(nrows=500, ncols=500, xmin=571823, xmax=616763, 
+#'             ymin=4423540, ymax=4453690)
+#'   crs(r) <- "epsg:9001"
+#' r[] <- runif(ncell(r), 1000, 2500)
 #'
-#'  # Postive values so, can apply any transformation    
-#'	for( i in c("norm", "rstd", "std", "stretch", "nl", "slog", "sr")) {
-#'	  print( raster.transformation(r, trans = i) ) 
-#'    }
+#'  # Positive values so, can apply any transformation    
+#'  for( i in c("norm", "rstd", "std", "stretch", "nl", "slog", "sr")) {
+#'    print( raster.transformation(r, trans = i) ) 
+#'  }
 #'
 #'  # Negative values so, can't transform using "nl", "slog" or "sr"
-#'	r[] <- runif(ncell(r), -1, 1)
+#'  r[] <- runif(ncell(r), -1, 1)
 #'    for( i in c("norm", "rstd", "std", "stretch", "nl", "slog", "sr")) {
 #'	  try( print( raster.transformation(r, trans = i) ) ) 
 #'    }
@@ -48,11 +49,12 @@
 #' @export raster.transformation
 raster.transformation <- function(x, trans = "norm", smin=0, smax=255) {
   slog <- function(x) { ifelse(abs(x) <= 1, 0, sign(x)*log10(abs(x)))}
-    rmin <- raster::cellStats(x, stat = "min", na.rm = TRUE)
-    rmax <- raster::cellStats(x, stat = "max", na.rm = TRUE)
-    rmean <- raster::cellStats(x, stat = "mean", na.rm = TRUE)
-    rsd <- raster::cellStats(x, stat = "sd", na.rm = TRUE)
-	
+   if (!inherits(x, "SpatRaster")) 
+    stop(deparse(substitute(x)), " must be a terra SpatRaster object")    
+	  rmin <- terra::global(x, "min", na.rm = TRUE)[,1]
+	  rmax <- terra::global(x, "max", na.rm = TRUE)[,1]
+  	  rmean <- terra::global(x, "mean", na.rm = TRUE)[,1]
+  	  rsd <- terra::global(x, "sd", na.rm = TRUE)[,1]	
   if( trans == "slog" && rmin < 0) {
     stop(" Minimum value < 0, cannot log transform")
   }
@@ -66,7 +68,6 @@ raster.transformation <- function(x, trans = "norm", smin=0, smax=255) {
     print(" Min value < 0, running row standardization instead")
     return( x / rmax )
   }
-  
   if( trans == "norm") {
     message("applying normalization transformation", "\n")
     return( ( x - rmin ) / ( rmax - rmin ) )
@@ -81,13 +82,13 @@ raster.transformation <- function(x, trans = "norm", smin=0, smax=255) {
 		   return( (x - rmin) * smax / (rmax - rmin) + smin )
 		    } else if ( trans == "nl") {
 			  message("applying log transformation", "\n")
-			  return(  raster::calc(x, fun=log) )
+			  return(  terra::app(x, fun=log) )
 			  } else if ( trans == "slog") {
-			    message("applying singned-log10 transformation", "\n")
-			    return(raster::calc(x, fun=slog) )
+			    message("applying signed-log10 transformation", "\n")
+			    return(terra::app(x, fun=slog) )
 			    } else if ( trans == "sr") {
-				  message("applying sqare-root transformation", "\n")
-			      return(  raster::calc(x, fun=sqrt) )		  
+				  message("applying square-root transformation", "\n")
+			      return( terra::app(x, fun=sqrt) )		  
 		          } else {
                     stop("Not a valid transformation") 
     }		
